@@ -2,7 +2,7 @@
 """
 
 
-__version__ = "1.2.1"
+__version__ = "1.3.2"
 __author__ = "Yngve Mardal Moe"
 __email__ = "yngve.m.moe@gmail.com"
 
@@ -10,6 +10,8 @@ __email__ = "yngve.m.moe@gmail.com"
 from group_lasso._group_lasso import (
     GroupLasso,
     LogisticGroupLasso,
-    MultinomialGroupLasso,
     BaseGroupLasso,
 )
+
+
+MultinomialGroupLasso = LogisticGroupLasso
