@@ -1,37 +1,36 @@
+import warnings
 from abc import ABC, abstractmethod
 from math import sqrt
 from numbers import Number
-import warnings
 
-import numpy.linalg as la
 import numpy as np
+import numpy.linalg as la
 from scipy import sparse
-from sklearn.utils import (
-    check_random_state,
-    check_array,
-    check_consistent_length,
-)
 from sklearn.base import (
     BaseEstimator,
-    TransformerMixin,
-    RegressorMixin,
     ClassifierMixin,
+    RegressorMixin,
+    TransformerMixin,
 )
 from sklearn.preprocessing import LabelBinarizer
+from sklearn.utils import (
+    check_array,
+    check_consistent_length,
+    check_random_state,
+)
 
+from group_lasso._fista import fista
 from group_lasso._singular_values import find_largest_singular_value
 from group_lasso._subsampling import subsample
-from group_lasso._fista import fista
-
 
 _DEBUG = False
 
 _OLD_REG_WARNING = """
 The behaviour has changed since v1.1.1, before then, a bug in the optimisation
-algorithm made it so the regularisation parameter was scaled by the largest 
+algorithm made it so the regularisation parameter was scaled by the largest
 eigenvalue of the covariance matrix.
 
-To use the old behaviour, initialise the class with the keyword argument 
+To use the old behaviour, initialise the class with the keyword argument
 `old_regularisation=True`.
 
 To supress this warning, initialise the class with the keyword argument
@@ -79,7 +78,7 @@ def _join_intercept(b, w):
 def _add_intercept_col(X):
     ones = np.ones([X.shape[0], 1])
     if sparse.issparse(X):
-        return sparse.hstack((ones, X))        
+        return sparse.hstack((ones, X))
     return np.hstack([ones, X])
 
 
@@ -93,6 +92,61 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
     The loss is optimised using the FISTA algorithm proposed in [2] with the
     generalised gradient-based restarting scheme proposed in [3].
 
+    Parameters
+    ----------
+    groups : Iterable
+        Iterable that specifies which group each column corresponds to.
+        For columns that should not be regularised, the corresponding
+        group index should either be None or negative. For example, the
+        list ``[1, 1, 1, 2, 2, -1]`` specifies that the first three
+        columns of the data matrix belong to the first group, the next
+        two columns belong to the second group and the last column should
+        not be regularised.
+    group_reg : float or iterable [default=0.05]
+        The regularisation coefficient(s) for the group sparsity penalty.
+        If ``group_reg`` is an iterable, then its length should be equal to
+        the number of groups.
+    l1_reg : float or iterable [default=0.05]
+        The regularisation coefficient for the coefficient sparsity
+        penalty.
+    n_iter : int [default=100]
+        The maximum number of iterations to perform
+    tol : float [default=1e-5]
+        The convergence tolerance. The optimisation algorithm
+        will stop once ||x_{n+1} - x_n|| < ``tol``.
+    scale_reg : str [in {"group_size", "none", "inverse_group_size"] or None
+        How to scale the group-wise regularisation coefficients. In the
+        original group lasso paper scaled the regularisation by the square
+        root of the elements in each group so that each variable has the
+        same effect on the regularisation. This is not sensible for dummy
+        encoded variables, as these always have either unit or zero norm.
+        ``scale_reg`` should therefore be None if all variables are dummy
+        variables. Finally, if the group size shouldn't be considered when
+        choosing variables, then inverse_group_size should be used instead
+        as that divide by the square root of the group size, removing the
+        dependence of group size on the regularisation strength.
+    subsampling_scheme : None, float, int or str [default=None]
+        The subsampling rate used for the gradient and singular value
+        computations. If it is a float, then it specifies the fraction
+        of rows to use in the computations. If it is an int, it
+        specifies the number of rows to use in the computation and if
+        it is a string, then it must be 'sqrt' and the number of rows used
+        in the computations is the square root of the number of rows
+        in X.
+    frobenius_lipschitz : bool [default=False]
+        Use the Frobenius norm to estimate the lipschitz coefficient of the
+        MSE loss. This works well for systems whose power iterations
+        converge slowly. If False, then subsampled power iterations are
+        used. Using the Frobenius approximation for the Lipschitz
+        coefficient might fail, and end up with all-zero weights.
+    fit_intercept : bool [default=True]
+        Whether to fit an intercept or not.
+    random_state : np.random.RandomState [default=None]
+        The random state used for initialisation of parameters.
+    warm_start : bool [default=False]
+        If true, then subsequent calls to fit will not re-initialise the
+        model parameters. This can speed up the hyperparameter search
+
     References
     ----------
     [1] Simon, N., Friedman, J., Hastie, T., & Tibshirani, R. (2013).
@@ -114,9 +168,10 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
         self,
         groups,
         group_reg=0.05,
-        l1_reg=0.05,
+        l1_reg=0.00,
         n_iter=100,
         tol=1e-5,
+        scale_reg="group_size",
         subsampling_scheme=None,
         fit_intercept=True,
         random_state=None,
@@ -124,54 +179,9 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
         old_regularisation=False,
         supress_warning=False,
     ):
-        """
-
-        Arguments
-        ---------
-        groups : Iterable
-            Iterable that specifies which group each column corresponds to.
-            For columns that should not be regularised, the corresponding
-            group index should either be None or negative. For example, the
-            list ``[1, 1, 1, 2, 2, -1]`` specifies that the first three
-            columns of the data matrix belong to the first group, the next
-            two columns belong to the second group and the last column should
-            not be regularised.
-        group_reg : float or iterable [default=0.05]
-            The regularisation coefficient(s) for the group sparsity penalty.
-            If ``group_reg`` is an iterable, then its length should be equal to
-            the number of groups.
-        l1_reg : float or iterable [default=0.05]
-            The regularisation coefficient for the coefficient sparsity
-            penalty.
-        n_iter : int [default=100]
-            The maximum number of iterations to perform
-        tol : float [default=1e-5]
-            The convergence tolerance. The optimisation algorithm
-            will stop once ||x_{n+1} - x_n|| < ``tol``.
-        subsampling_scheme : None, float, int or str [default=None]
-            The subsampling rate used for the gradient and singular value
-            computations. If it is a float, then it specifies the fraction
-            of rows to use in the computations. If it is an int, it
-            specifies the number of rows to use in the computation and if
-            it is a string, then it must be 'sqrt' and the number of rows used
-            in the computations is the square root of the number of rows
-            in X.
-        frobenius_lipschitz : bool [default=False]
-            Use the Frobenius norm to estimate the lipschitz coefficient of the
-            MSE loss. This works well for systems whose power iterations
-            converge slowly. If False, then subsampled power iterations are
-            used. Using the Frobenius approximation for the Lipschitz
-            coefficient might fail, and end up with all-zero weights.
-        fit_intercept : bool [default=True]
-            Whether to fit an intercept or not.
-        random_state : np.random.RandomState [default=None]
-            The random state used for initialisation of parameters.
-        warm_start : bool [default=False]
-            If true, then subsequent calls to fit will not re-initialise the
-            model parameters. This can speed up the hyperparameter search
-        """
         self.groups = groups
         self.group_reg = group_reg
+        self.scale_reg = scale_reg
         self.l1_reg = l1_reg
         self.n_iter = n_iter
         self.tol = tol
@@ -195,17 +205,36 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
         regulariser += self.l1_reg * la.norm(w.ravel(), 1)
         return regulariser
 
+    def _get_reg_strength(self, group, reg):
+        """Get the regularisation coefficient for one group.
+        """
+        scale_reg = str(self.scale_reg).lower()
+        if scale_reg == "group_size":
+            scale = sqrt(group.sum())
+        elif scale_reg == "none":
+            scale = 1
+        elif scale_reg == "inverse_group_size":
+            scale = 1 / sqrt(group.sum())
+        else:
+            raise ValueError(
+                '``scale_reg`` must be equal to "group_size",'
+                ' "inverse_group_size" or "none"'
+            )
+        return reg * scale
+
     def _get_reg_vector(self, reg):
         """Get the group-wise regularisation coefficients from ``reg``.
         """
         if isinstance(reg, Number):
-            reg = [reg * sqrt(group.sum()) for group in self.groups_]
+            reg = [
+                self._get_reg_strength(group, reg) for group in self.groups_
+            ]
         else:
             reg = list(reg)
         return reg
 
     @abstractmethod
-    def _unregularised_loss(self, X, y, w):  #  pragma: nocover
+    def _unregularised_loss(self, X, y, w):  # pragma: nocover
         """The unregularised reconstruction loss.
         """
         raise NotImplementedError
@@ -213,8 +242,8 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
     def _loss(self, X, y, w):
         """The group-lasso regularised loss.
 
-        Arguments
-        ---------
+        Parameters
+        ----------
         X : np.ndarray
             Data matrix, ``X.shape == (num_datapoints, num_features)``
         y : np.ndarray
@@ -229,15 +258,17 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
     def loss(self, X, y):
         """The group-lasso regularised loss with the current coefficients
 
-        Arguments
-        ---------
+        Parameters
+        ----------
         X : np.ndarray
             Data matrix, ``X.shape == (num_datapoints, num_features)``
         y : np.ndarray
             Target vector/matrix, ``y.shape == (num_datapoints, num_targets)``,
             or ``y.shape == (num_datapoints,)``
         """
-        return self._loss(X, y, self.coef_)
+        X = _add_intercept_col(X)
+        w = _join_intercept(self.intercept_, self.coef_)
+        return self._loss(X, y, w)
 
     @abstractmethod
     def _compute_lipschitz(self, X, y):  # pragma: nocover
@@ -312,7 +343,7 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
                     )
                 )
 
-        weights = np.concatenate([self.intercept_, self.coef_])
+        weights = _join_intercept(self.intercept_, self.coef_)
         weights = fista(
             weights,
             grad=self._unregularised_gradient,
@@ -340,7 +371,7 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
         """Ensure that the inputs are valid and prepare them for fit.
         """
         check_consistent_length(X, y)
-        X = check_array(X, accept_sparse='csr')
+        X = check_array(X, accept_sparse="csr")
         y = check_array(y)
         if len(y.shape) == 1:
             y = y.reshape(-1, 1)
@@ -348,14 +379,14 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
         # Add the intercept column and compute Lipschitz bound the correct way
         if self.fit_intercept:
             X = _add_intercept_col(X)
-            X = check_array(X, accept_sparse='csr')
+            X = check_array(X, accept_sparse="csr")
 
         if lipschitz is None:
             lipschitz = self._compute_lipschitz(X, y)
 
         if not self.fit_intercept:
             X = _add_intercept_col(X)
-            X = check_array(X, accept_sparse='csr')
+            X = check_array(X, accept_sparse="csr")
 
         return X, y, lipschitz
 
@@ -370,7 +401,7 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
         self.group_reg_vector_ = self._get_reg_vector(self.group_reg)
         self.losses_ = []
 
-        if not self.warm_start or not hasattr(self, coef_):
+        if not self.warm_start or not hasattr(self, "coef_"):
             self.coef_ = self.random_state_.standard_normal(
                 (X.shape[1] - 1, y.shape[1])
             )
@@ -402,17 +433,35 @@ class BaseGroupLasso(ABC, BaseEstimator, TransformerMixin):
     def sparsity_mask(self):
         """A boolean mask indicating whether features are used in prediction.
         """
-        coef_ = self.coef_.mean(1)
+        warnings.warn(
+            "This property is discontinued, use sparsity_mask_ instead of sparsity_mask."
+        )
+        return self.sparsity_mask_
+
+    @property
+    def sparsity_mask_(self):
+        """A boolean mask indicating whether features are used in prediction.
+        """
+        if len(self.groups.shape) == 1 or self.groups.shape[1] == 1:
+            coef_ = self.coef_.mean(1)
+        else:
+            coef_ = self.coef_
         mean_abs_coef = abs(coef_.mean())
 
         return np.abs(coef_) > 1e-10 * mean_abs_coef
 
+    @property
+    def chosen_groups_(self):
+        """A set of the coosen group ids.
+        """
+        return set(np.unique(self.groups[self.sparsity_mask_]))
+
     def transform(self, X):
         """Remove columns corresponding to zero-valued coefficients.
         """
         if sparse.issparse(X):
-            X = check_array(X, accept_sparse='csc')
-        return X[:, self.sparsity_mask]
+            X = check_array(X, accept_sparse="csc")
+        return X[:, self.sparsity_mask_]
 
     def fit_transform(self, X, y, lipschitz=None):
         """Fit a group lasso model to X and y and remove unused columns from X
@@ -452,6 +501,61 @@ class GroupLasso(BaseGroupLasso, RegressorMixin):
     the viable features and that the output is fed into another regression
     algorithm, such as RidgeRegression in scikit-learn.
 
+    Parameters
+    ----------
+    groups : Iterable
+        Iterable that specifies which group each column corresponds to.
+        For columns that should not be regularised, the corresponding
+        group index should either be None or negative. For example, the
+        list ``[1, 1, 1, 2, 2, -1]`` specifies that the first three
+        columns of the data matrix belong to the first group, the next
+        two columns belong to the second group and the last column should
+        not be regularised.
+    group_reg : float or iterable [default=0.05]
+        The regularisation coefficient(s) for the group sparsity penalty.
+        If ``group_reg`` is an iterable, then its length should be equal to
+        the number of groups.
+    l1_reg : float or iterable [default=0.05]
+        The regularisation coefficient for the coefficient sparsity
+        penalty.
+    n_iter : int [default=100]
+        The maximum number of iterations to perform
+    tol : float [default=1e-5]
+        The convergence tolerance. The optimisation algorithm
+        will stop once ||x_{n+1} - x_n|| < ``tol``.
+    scale_reg : str [in {"group_size", "none", "inverse_group_size"] or None
+        How to scale the group-wise regularisation coefficients. In the
+        original group lasso paper scaled the regularisation by the square
+        root of the elements in each group so that each variable has the
+        same effect on the regularisation. This is not sensible for dummy
+        encoded variables, as these always have either unit or zero norm.
+        ``scale_reg`` should therefore be None if all variables are dummy
+        variables. Finally, if the group size shouldn't be considered when
+        choosing variables, then inverse_group_size should be used instead
+        as that divide by the square root of the group size, removing the
+        dependence of group size on the regularisation strength.
+    subsampling_scheme : None, float, int or str [default=None]
+        The subsampling rate used for the gradient and singular value
+        computations. If it is a float, then it specifies the fraction
+        of rows to use in the computations. If it is an int, it
+        specifies the number of rows to use in the computation and if
+        it is a string, then it must be 'sqrt' and the number of rows used
+        in the computations is the square root of the number of rows
+        in X.
+    frobenius_lipschitz : bool [default=False]
+        Use the Frobenius norm to estimate the lipschitz coefficient of the
+        MSE loss. This works well for systems whose power iterations
+        converge slowly. If False, then subsampled power iterations are
+        used. Using the Frobenius approximation for the Lipschitz
+        coefficient might fail, and end up with all-zero weights.
+    fit_intercept : bool [default=True]
+        Whether to fit an intercept or not.
+    random_state : np.random.RandomState [default=None]
+        The random state used for initialisation of parameters.
+    warm_start : bool [default=False]
+        If true, then subsequent calls to fit will not re-initialise the
+        model parameters. This can speed up the hyperparameter search
+
     References
     ----------
     [1] Simon, N., Friedman, J., Hastie, T., & Tibshirani, R. (2013).
@@ -474,76 +578,36 @@ class GroupLasso(BaseGroupLasso, RegressorMixin):
         l1_reg=0.05,
         n_iter=100,
         tol=1e-5,
+        scale_reg="group_size",
         subsampling_scheme=None,
         fit_intercept=True,
         frobenius_lipschitz=False,
         random_state=None,
         warm_start=False,
+        old_regularisation=False,
+        supress_warning=False,
     ):
-        """
-
-        Arguments
-        ---------
-        groups : Iterable
-            Iterable that specifies which group each column corresponds to.
-            For columns that should not be regularised, the corresponding
-            group index should either be None or negative. For example, the
-            list ``[1, 1, 1, 2, 2, -1]`` specifies that the first three
-            columns of the data matrix belong to the first group, the next
-            two columns belong to the second group and the last column should
-            not be regularised.
-        group_reg : float or iterable [default=0.05]
-            The regularisation coefficient(s) for the group sparsity penalty.
-            If ``group_reg`` is an iterable, then its length should be equal to
-            the number of groups.
-        l1_reg : float or iterable [default=0.05]
-            The regularisation coefficient for the coefficient sparsity
-            penalty.
-        n_iter : int [default=100]
-            The maximum number of iterations to perform
-        tol : float [default=1e-5]
-            The convergence tolerance. The optimisation algorithm
-            will stop once ||x_{n+1} - x_n|| < ``tol``.
-        subsampling_scheme : None, float, int or str [default=None]
-            The subsampling rate used for the gradient and singular value
-            computations. If it is a float, then it specifies the fraction
-            of rows to use in the computations. If it is an int, it
-            specifies the number of rows to use in the computation and if
-            it is a string, then it must be 'sqrt' and the number of rows used
-            in the computations is the square root of the number of rows
-            in X.
-        frobenius_lipschitz : bool [default=False]
-            Use the Frobenius norm to estimate the lipschitz coefficient of the
-            MSE loss. This works well for systems whose power iterations
-            converge slowly. If False, then subsampled power iterations are
-            used. Using the Frobenius approximation for the Lipschitz
-            coefficient might fail, and end up with all-zero weights.
-        fit_intercept : bool [default=True]
-            Whether to fit an intercept or not.
-        random_state : np.random.RandomState [default=None]
-            The random state used for initialisation of parameters.
-        warm_start : bool [default=False]
-            If true, then subsequent calls to fit will not re-initialise the
-            model parameters. This can speed up the hyperparameter search
-        """
         super().__init__(
             groups=groups,
             l1_reg=l1_reg,
             group_reg=group_reg,
             n_iter=n_iter,
             tol=tol,
+            scale_reg=scale_reg,
             subsampling_scheme=subsampling_scheme,
             fit_intercept=fit_intercept,
             random_state=random_state,
             warm_start=warm_start,
+            old_regularisation=old_regularisation,
+            supress_warning=supress_warning,
         )
         self.frobenius_lipchitz = frobenius_lipschitz
 
     def fit(self, X, y, lipschitz=None):
         """Fit a group lasso regularised linear regression model.
 
-        Arguments
-        ---------
+        Parameters
+        ----------
         X : np.ndarray
             Data matrix
         y : np.ndarray
@@ -573,7 +637,7 @@ class GroupLasso(BaseGroupLasso, RegressorMixin):
         num_rows, num_cols = X.shape
         if self.frobenius_lipchitz:
             if sparse.issparse(X):
-                return sparse.linalg.norm(X, "fro")**2/num_rows
+                return sparse.linalg.norm(X, "fro") ** 2 / num_rows
             return la.norm(X, "fro") ** 2 / num_rows
 
         s_max = find_largest_singular_value(
@@ -585,97 +649,6 @@ class GroupLasso(BaseGroupLasso, RegressorMixin):
         return SSE_lipschitz / num_rows
 
 
-def _sigmoid(x):
-    return 1 / (1 + np.exp(-x))
-
-
-def _logit(X, w):
-    return X @ w
-
-
-def _logistic_proba(X, w):
-    return _sigmoid(_logit(X, w))
-
-
-def _logistic_cross_entropy(X, y, w):
-    p = _logistic_proba(X, w)
-    return -(y * np.log(p) + (1 - y) * np.log(1 - p))
-
-
-class LogisticGroupLasso(BaseGroupLasso, ClassifierMixin):
-    """Sparse group lasso regularised single-class logistic regression.
-
-    This class implements the Sparse Group Lasso [1] regularisation for
-    logistic regression with a cross entropy penalty.
-
-    This class is implemented as both a regressor and a transformation.
-    If the ``transform`` method is called, then the columns of the input
-    that correspond to zero-valued regression coefficients are dropped.
-
-    The loss is optimised using the FISTA algorithm proposed in [2] with the
-    generalised gradient-based restarting scheme proposed in [3]. This
-    algorithm is not as accurate as a few other optimisation algorithms,
-    but it is extremely efficient and does recover the sparsity patterns.
-    We therefore reccomend that this class is used as a transformer to select
-    the viable features and that the output is fed into another classification
-    algorithm, such as LogisticRegression in scikit-learn.
-
-    References
-    ----------
-    [1] Simon, N., Friedman, J., Hastie, T., & Tibshirani, R. (2013).
-    A sparse-group lasso. Journal of Computational and Graphical
-    Statistics, 22(2), 231-245.
-
-    [2] Beck A, Teboulle M. (2009). A fast iterative shrinkage-thresholding
-    algorithm for linear inverse problems. SIAM journal on imaging
-    sciences. 2009 Mar 4;2(1):183-202.
-
-    [3] O’Donoghue B, Candes E. (2015) Adaptive restart for accelerated
-    gradient schemes. Foundations of computational mathematics.
-    Jun 1;15(3):715-32.
-    """
-
-    def _compute_proba(self, X, w):
-        return _sigmoid(X @ w)
-
-    def _unregularised_loss(self, X, y, w):
-        X_, y_ = self.subsample(X, y)
-        return _logistic_cross_entropy(X_, y_, w).sum() / X.shape[0]
-
-    def _grad(self, X, y, w):
-        X_, y_ = self.subsample(X, y)
-        p = _logistic_proba(X_, w)
-        return X_.T @ (p - y_) / X_.shape[0]
-
-    def _compute_lipschitz(self, X, y):
-        if sparse.issparse(X):
-            norm = sparse.linalg.norm(X, "fro")
-        else:
-            norm = la.norm(X, "fro")
-        return np.sqrt(12) * norm / X.shape[0]
-
-    def predict_proba(self, X):
-        return _logistic_proba(X, self.coef_)
-
-    def predict(self, X):
-        """Predict using the linear model.
-        """
-        return self.predict_proba(X) >= 0.5
-
-    def fit(self, X, y, lipschitz=None):
-        if y.ndim == 2 and y.shape[1] > 1:
-            n = y.shape[1]
-            warnings.warn(
-                (
-                    "You have passed {n} targets to a single class classifier."
-                    " This will simply train {n} different models meaning that"
-                    " multiple classes can be predicted as true at once."
-                ).format(n=n)
-            )
-
-        super().fit(X, y, lipschitz=lipschitz)
-
-
 def _softmax(logit):
     logit = logit - logit.max(1, keepdims=True)
     expl = np.exp(logit)
@@ -683,7 +656,7 @@ def _softmax(logit):
 
 
 def _softmax_proba(X, W):
-    return _softmax(_logit(X, W))
+    return _softmax(X @ W)
 
 
 def _softmax_cross_entropy(X, Y, W):
@@ -691,18 +664,11 @@ def _softmax_cross_entropy(X, Y, W):
     return -np.sum(Y * np.log(P))
 
 
-def _one_hot_encode(y):
-    if y.ndim == 1:
-        y = LabelBinarizer().fit_transform(y[:, np.newaxis])
-    return y
-
-
-class MultinomialGroupLasso(BaseGroupLasso, ClassifierMixin):
+class LogisticGroupLasso(BaseGroupLasso, ClassifierMixin):
     """Sparse group lasso regularised multi-class logistic regression.
 
     This class implements the Sparse Group Lasso [1] regularisation for
-    multinomial regression (also known as multi-class logistic regression)
-    with a cross entropy penalty.
+    multi-class logistic regression with a cross entropy penalty.
 
     This class is implemented as both a regressor and a transformation.
     If the ``transform`` method is called, then the columns of the input
@@ -716,6 +682,61 @@ class MultinomialGroupLasso(BaseGroupLasso, ClassifierMixin):
     the viable features and that the output is fed into another classification
     algorithm, such as LogisticRegression in scikit-learn.
 
+    Parameters
+    ----------
+    groups : Iterable
+        Iterable that specifies which group each column corresponds to.
+        For columns that should not be regularised, the corresponding
+        group index should either be None or negative. For example, the
+        list ``[1, 1, 1, 2, 2, -1]`` specifies that the first three
+        columns of the data matrix belong to the first group, the next
+        two columns belong to the second group and the last column should
+        not be regularised.
+    group_reg : float or iterable [default=0.05]
+        The regularisation coefficient(s) for the group sparsity penalty.
+        If ``group_reg`` is an iterable, then its length should be equal to
+        the number of groups.
+    l1_reg : float or iterable [default=0.05]
+        The regularisation coefficient for the coefficient sparsity
+        penalty.
+    n_iter : int [default=100]
+        The maximum number of iterations to perform
+    tol : float [default=1e-5]
+        The convergence tolerance. The optimisation algorithm
+        will stop once ||x_{n+1} - x_n|| < ``tol``.
+    scale_reg : str [in {"group_size", "none", "inverse_group_size"] or None
+        How to scale the group-wise regularisation coefficients. In the
+        original group lasso paper scaled the regularisation by the square
+        root of the elements in each group so that each variable has the
+        same effect on the regularisation. This is not sensible for dummy
+        encoded variables, as these always have either unit or zero norm.
+        ``scale_reg`` should therefore be None if all variables are dummy
+        variables. Finally, if the group size shouldn't be considered when
+        choosing variables, then inverse_group_size should be used instead
+        as that divide by the square root of the group size, removing the
+        dependence of group size on the regularisation strength.
+    subsampling_scheme : None, float, int or str [default=None]
+        The subsampling rate used for the gradient and singular value
+        computations. If it is a float, then it specifies the fraction
+        of rows to use in the computations. If it is an int, it
+        specifies the number of rows to use in the computation and if
+        it is a string, then it must be 'sqrt' and the number of rows used
+        in the computations is the square root of the number of rows
+        in X.
+    frobenius_lipschitz : bool [default=False]
+        Use the Frobenius norm to estimate the lipschitz coefficient of the
+        MSE loss. This works well for systems whose power iterations
+        converge slowly. If False, then subsampled power iterations are
+        used. Using the Frobenius approximation for the Lipschitz
+        coefficient might fail, and end up with all-zero weights.
+    fit_intercept : bool [default=True]
+        Whether to fit an intercept or not.
+    random_state : np.random.RandomState [default=None]
+        The random state used for initialisation of parameters.
+    warm_start : bool [default=False]
+        If true, then subsequent calls to fit will not re-initialise the
+        model parameters. This can speed up the hyperparameter search
+
     References
     ----------
     [1] Simon, N., Friedman, J., Hastie, T., & Tibshirani, R. (2013).
@@ -738,60 +759,17 @@ class MultinomialGroupLasso(BaseGroupLasso, ClassifierMixin):
         l1_reg=0.05,
         n_iter=100,
         tol=1e-5,
+        scale_reg="group_size",
         subsampling_scheme=None,
         fit_intercept=True,
         random_state=None,
         warm_start=False,
+        old_regularisation=False,
+        supress_warning=False,
     ):
-        """
-
-        Arguments
-        ---------
-        groups : Iterable
-            Iterable that specifies which group each column corresponds to.
-            For columns that should not be regularised, the corresponding
-            group index should either be None or negative. For example, the
-            list ``[1, 1, 1, 2, 2, -1]`` specifies that the first three
-            columns of the data matrix belong to the first group, the next
-            two columns belong to the second group and the last column should
-            not be regularised.
-        group_reg : float or iterable [default=0.05]
-            The regularisation coefficient(s) for the group sparsity penalty.
-            If ``group_reg`` is an iterable, then its length should be equal to
-            the number of groups.
-        l1_reg : float or iterable [default=0.05]
-            The regularisation coefficient for the coefficient sparsity
-            penalty.
-        n_iter : int [default=100]
-            The maximum number of iterations to perform
-        tol : float [default=1e-5]
-            The convergence tolerance. The optimisation algorithm
-            will stop once ||x_{n+1} - x_n|| < ``tol``.
-        subsampling_scheme : None, float, int or str [default=None]
-            The subsampling rate used for the gradient and singular value
-            computations. If it is a float, then it specifies the fraction
-            of rows to use in the computations. If it is an int, it
-            specifies the number of rows to use in the computation and if
-            it is a string, then it must be 'sqrt' and the number of rows used
-            in the computations is the square root of the number of rows
-            in X.
-        frobenius_lipschitz : bool [default=False]
-            Use the Frobenius norm to estimate the lipschitz coefficient of the
-            MSE loss. This works well for systems whose power iterations
-            converge slowly. If False, then subsampled power iterations are
-            used. Using the Frobenius approximation for the Lipschitz
-            coefficient might fail, and end up with all-zero weights.
-        fit_intercept : bool [default=True]
-            Whether to fit an intercept or not.
-        random_state : np.random.RandomState [default=None]
-            The random state used for initialisation of parameters.
-        warm_start : bool [default=False]
-            If true, then subsequent calls to fit will not re-initialise the
-            model parameters. This can speed up the hyperparameter search
-        """
         if subsampling_scheme is not None:
             warnings.warn(
-                "Subsampling is not stable for multinomial group lasso."
+                "Subsampling is not stable for logistic regression group lasso."
             )
         super().__init__(
             groups=groups,
@@ -799,22 +777,23 @@ class MultinomialGroupLasso(BaseGroupLasso, ClassifierMixin):
             l1_reg=l1_reg,
             n_iter=n_iter,
             tol=tol,
+            scale_reg=scale_reg,
             subsampling_scheme=subsampling_scheme,
             fit_intercept=fit_intercept,
             random_state=random_state,
             warm_start=warm_start,
+            old_regularisation=old_regularisation,
+            supress_warning=supress_warning,
         )
 
     def _compute_proba(self, X, w):
         return _softmax_proba(X, w)
 
     def _unregularised_loss(self, X, y, w):
-        y = _one_hot_encode(y)
         X_, y_ = self.subsample(X, y)
         return _softmax_cross_entropy(X_, y_, w).sum() / X.shape[0]
 
     def _grad(self, X, y, w):
-        y = _one_hot_encode(y)
         X_, y_ = self.subsample(X, y)
         p = _softmax_proba(X_, w)
 
@@ -830,19 +809,30 @@ class MultinomialGroupLasso(BaseGroupLasso, ClassifierMixin):
         return 2 * C ** (1 / 4) * norm / X.shape[0]
 
     def predict_proba(self, X):
-        return _softmax_proba(X, self.coef_)
+        return _softmax_proba(X, self.coef_).T
 
     def predict(self, X):
         """Predict using the linear model.
         """
-        return np.argmax(self.predict_proba(X), axis=1)
+        return np.argmax(self.predict_proba(X), axis=0)[:, np.newaxis]
+
+    def _encode(self, y):
+        """One-hot encoding for the labels.
+        """
+        y = self.label_binarizer_.transform(y)
+        if y.shape[1] == 1:
+            ones = np.ones((y.shape[0], 1))
+            y = np.hstack(((ones - y.sum(1, keepdims=True)), y,))
+        return y
 
     def _prepare_dataset(self, X, y, lipschitz):
         """Ensure that the inputs are valid and prepare them for fit.
         """
-        y = _one_hot_encode(y)
+        self.label_binarizer_ = LabelBinarizer()
+        self.label_binarizer_.fit(y)
+        y = self._encode(y)
         check_consistent_length(X, y)
-        X = check_array(X, accept_sparse='csr')
+        X = check_array(X, accept_sparse="csr")
         check_array(y, ensure_2d=False)
         if set(np.unique(y)) != {0, 1}:
             raise ValueError(
@@ -853,13 +843,13 @@ class MultinomialGroupLasso(BaseGroupLasso, ClassifierMixin):
         # Add the intercept column and compute Lipschitz bound the correct way
         if self.fit_intercept:
             X = _add_intercept_col(X)
-            X = check_array(X, accept_sparse='csr')
+            X = check_array(X, accept_sparse="csr")
 
         if lipschitz is None:
             lipschitz = self._compute_lipschitz(X, y)
 
         if not self.fit_intercept:
             X = _add_intercept_col(X)
-            X = check_array(X, accept_sparse='csr')
+            X = check_array(X, accept_sparse="csr")
 
         return X, y, lipschitz
