@@ -1,3 +1,5 @@
+"""
+"""
 import numpy as np
 
 
@@ -20,4 +22,9 @@ def extract_ohe_groups(onehot_encoder):
         )
 
     categories = onehot_encoder.categories_
-    return np.concatenate([group*np.ones_like(category) for group, category in enumerate(categories)])
+    return np.concatenate(
+        [
+            group*np.ones_like(category) 
+            for group, category in enumerate(categories)
+        ]
+    )
